#!/usr/bin/env python
# coding: utf-8

# In[1]:

import geopandas as gpd
import pandas as pd
from rasterstats import zonal_stats
import rtree
import pygeos

# In[2]:

WORK_DIR = ''
ISOCHRONES_FILE = 'isochrones/ssd_heal_med_pt_s0_pp_C19PP_isochrone_driving_30.json'
POP_TIF = 'data/worldpop/SSD_population_v2_0_gridded/SSD_population_v2_0_gridded_population_fixed.tif'
# Isochrone and point buffer distance in UTM meters
BUFFER_DISTANCE = 1
POINT_BUFFER_DISTANCE = 4000
LIMIT = False
SAVE_OUTPUTS = True
OUTPUT_NAME = 's_sudan_buffered_1m4km'
SCHOOLS_FILE = 'data/healthsites/south_sudan/ssd_heal_med_pt_s0_pp_C19PP.geojson'

# In[3]:


# Load only some features for testing if Limit is defined
print("Reading input file "+ISOCHRONES_FILE)
isochrones_gdf = gpd.read_file(ISOCHRONES_FILE)[
    :LIMIT] if LIMIT else gpd.read_file(ISOCHRONES_FILE)


# In[4]:
isochrones_gdf.plot()

# In[6]
# Pick the middle UTM zone for buffering


# def repair_nodata(image):
#     ds = gdal.Open(image)
#     band1 = ds.GetRasterBand(1).ReadAsArray()
#     # Set all negative values to 0 for summing
#     band1[band1 < 0] = 0
#     print("Sum of raster: "+str(band1.sum()))
#     return band1


def set_utm_from_gdf(gdf):
    # Other approaches:
    # Option 1: https://pyproj4.github.io/pyproj/stable/examples.html#find-utm-crs-by-latitude-and-longitude
    # Option 2: https://timothyrenner.github.io/talks/20171128-geo-pydata/#/transverse-mercator
    # https://pyproj4.github.io/pyproj/stable/api/aoi.html#pyproj.aoi.AreaOfInterest
    from pyproj.aoi import AreaOfInterest
    from pyproj.database import query_utm_crs_info
    bounds = gdf.total_bounds
    aou = AreaOfInterest(west_lon_degree=bounds[0], south_lat_degree=bounds[1],
                         east_lon_degree=bounds[2], north_lat_degree=bounds[3])
    zones = query_utm_crs_info(area_of_interest=aou)
    middle = zones[int(len(zones)/2)]
    print("Central zone projection code: "+middle.code)
    return gdf.to_crs(epsg=middle.code)


isochrones_gdf = set_utm_from_gdf(isochrones_gdf)

# In[190]:
print("Reading input points "+SCHOOLS_FILE)
points_gdf = gpd.read_file(SCHOOLS_FILE)  # .set_crs(epsg=4326) # need for csv
print("Total features: "+str(len(points_gdf)))
# project to same crs as isochrones for projection
points_buffer_gdf = points_gdf.to_crs(isochrones_gdf.crs)
points_buffer_gdf.plot()

# In[191]:

print(f'Buffering point gemetry by {POINT_BUFFER_DISTANCE} meters')
points_buffer_gdf.geometry = points_buffer_gdf.buffer(POINT_BUFFER_DISTANCE)
points_buffer_gdf.plot()

# Buffer geometries to incorporate walking time to roads and fix invalid isochrones.
print(f'Buffering geometry by {BUFFER_DISTANCE} meters')
isochrones_gdf.geometry = isochrones_gdf.geometry.buffer(BUFFER_DISTANCE)


# In[13]:
isochrones_gdf.plot()
# Merge the point buffers and isochrones before dissolving
isochrones_gdf = pd.concat([isochrones_gdf, points_buffer_gdf])
print(len(isochrones_gdf))

# In[14]:
isochrones_gdf.plot()
# In[14]:
# Dissolve and reproject back
isochrones_dissolved = isochrones_gdf.dissolve().to_crs(epsg=4326)

if SAVE_OUTPUTS:
    # Modify here for geopackage outputs
    output_path = f'output/{OUTPUT_NAME}.shp'
    print("Saving layers to file: "+output_path)
    isochrones_dissolved.to_file(output_path)

# In[ ]:
print('Processing zonal stats')
# This will error when using https://wopr.worldpop.org/?SSD/Population unless NODATA values are repaired. For quick fix, set to 0 using QGIS or GDAL
pop_count = zonal_stats(isochrones_dissolved, POP_TIF, stats="sum")
print(f'Pop count: {str(pop_count)}')
